/*---[ preferences.c ]------------------------------------------------
 * Copyright (C) 2000-2002 Tomas Junnonen (majix@sci.fi)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * The preferences notebook window
 *--------------------------------------------------------------------*/
#include <stdio.h>
#include <errno.h>

#include "globals.h"
#include "firestarter.h"
#include "preferences.h"
#include "wizard.h"
#include "wizard-choices.h"
#include "scriptwriter.h"

enum 
{
	GENERAL_PAGE,
	FILTERS_PAGE,
	EXTERNAL_PAGE,
	SHARING_PAGE,
	SERVICES_PAGE,
	ICMP_PAGE,
	TOS_PAGE,
	ADVANCED_PAGE,
};


static GtkWidget *sections_clist;

Wizard *wizard;

GtkWidget *scriptentry;
GtkWidget *logentry;

GtkWidget *startup;
GtkWidget *stopatexit;
GtkWidget *resolve_names;
GtkWidget *tray_mode;

GtkWidget *redundant_filter;
GtkWidget *sep_filter;

GtkWidget *blockbroadcast;
GtkWidget *selectdeny;
GtkWidget *selectreject;

/* [ save_preferences ]
 * Save the preferences to the gnome registry
 */
static void
save_preferences (GnomePropertyBox *property_box, gint page_num, gint num)
{
	if (page_num == GENERAL_PAGE) {
		gnome_config_set_bool ("/firestarter/Program/startup",
			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (startup)));

		gnome_config_set_bool ("/firestarter/Program/shutdown",
			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (stopatexit)));

		gnome_config_set_bool ("/firestarter/Program/resolve_names",
			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (resolve_names)));

		gnome_config_set_bool ("/firestarter/Program/tray_mode",
			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tray_mode)));
	}

	if (page_num == FILTERS_PAGE) {
		gnome_config_set_bool ("/firestarter/Program/redundant_filter",
			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (redundant_filter)));

		gnome_config_set_bool ("/firestarter/Program/sep_filter",
			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (sep_filter)));
	}

	if (page_num == ADVANCED_PAGE) {
		gnome_config_set_bool ("/firestarter/Wizard/blockbroadcast",
			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (blockbroadcast)));

		gnome_config_set_bool ("/firestarter/Wizard/selectdeny",
			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (selectdeny)));

		gnome_config_set_bool ("/firestarter/Wizard/selectreject",
			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (selectreject)));
	}

	/* On the last run the page is -1 */
	if (page_num == -1) {
		FirewallStatus state;

		gnome_config_sync ();
		save_choices (wizard); /* This saves the wizard imported widgets states */
		write_script (wizard);

		state = get_current_status ();
		if (state == STATUS_RUNNING || state == STATUS_HIT)		
			start_firewall ();
	}
}

/* [ create_general_page ]
 * Create the general property page
 */
static GtkWidget*
create_general_page (GtkWidget *pbox)
{
	GtkWidget *table;
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *label;

	table = gtk_table_new (1, 1, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE (table), 0);
	gtk_table_set_col_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);

	vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);

	gtk_table_attach (GTK_TABLE (table), vbox, 0, 1, 0, 1,
			  GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);

	label = gtk_label_new (_(
		"The general program settings let you configure the overall behavior\n"
		"of the program.\n"));
	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);

	startup = gtk_check_button_new_with_label (_(
		"Start firewall on program startup"));
	gtk_tooltips_set_tip (Firestarter.ttips, startup, _(
		"Check this and the firewall will be started when the "
		"program loads."), "");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (startup),
		gnome_config_get_bool ("/firestarter/Program/startup=TRUE"));
	gtk_box_pack_start (GTK_BOX (vbox), startup, FALSE, FALSE, 0);

	stopatexit = gtk_check_button_new_with_label (_(
		"Stop firewall on program exit"));
	gtk_tooltips_set_tip (Firestarter.ttips, stopatexit, _(
		"Check this and the firewall will be stopped when you exit "
		"the program."), "");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (stopatexit),
		gnome_config_get_bool ("/firestarter/Program/shutdown=FALSE"));
	gtk_box_pack_start (GTK_BOX (vbox), stopatexit, FALSE, FALSE, 0);

	resolve_names = gtk_check_button_new_with_label (_("Autoresolve IP numbers to hostnames for hits"));
	gtk_tooltips_set_tip (Firestarter.ttips, resolve_names, _(
		"Check this and instead of the IP number you will see a hostname"
		" in the Hits view. This option uses some network bandwidth."), "");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (resolve_names),
		gnome_config_get_bool ("/firestarter/Program/resolve_names=FALSE"));
	gtk_box_pack_start (GTK_BOX (vbox), resolve_names, FALSE, FALSE, 0);

	tray_mode = gtk_check_button_new_with_label (_("System tray operation mode"));
	gtk_tooltips_set_tip (Firestarter.ttips, tray_mode, _(
		"When closing the program window in tray mode the program is hid "
		"instead of exited and can be restored by clicking the Firestarter system tray icon."), "");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tray_mode),
		gnome_config_get_bool ("/firestarter/Program/tray_mode=FALSE"));
	gtk_box_pack_start (GTK_BOX (vbox), tray_mode, FALSE, FALSE, 0);


	hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

	g_signal_connect_object (G_OBJECT (startup), "toggled",
		G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);
	g_signal_connect_object (G_OBJECT (stopatexit), "toggled",
		G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);
	g_signal_connect_object (G_OBJECT (resolve_names), "toggled",
		G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);
	g_signal_connect_object (G_OBJECT (tray_mode), "toggled",
		G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);

	return table;
}

/* [ create_filters_page ]
 * Create the general property page
 */
static GtkWidget*
create_filters_page (GtkWidget *pbox)
{
	GtkWidget *table;
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *label;

	table = gtk_table_new (1, 1, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE (table), 0);
	gtk_table_set_col_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);

	vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);

	gtk_table_attach (GTK_TABLE (table), vbox, 0, 1, 0, 1,
			  GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);

	label = gtk_label_new (_(
		"The filtering settings let you weed out excessive information\n"
		"from the Hits view.\n"));
	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);

	redundant_filter = gtk_check_button_new_with_label (_(
		"Filter out redundant hits"));
	gtk_tooltips_set_tip (Firestarter.ttips, redundant_filter, _(
		"Check this and the Hits view will not show identical "
		"hits that follow each other."), "");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (redundant_filter),
		gnome_config_get_bool ("/firestarter/Program/redundant_filter=TRUE"));
	gtk_box_pack_start (GTK_BOX (vbox), redundant_filter, FALSE, FALSE, 0);

	sep_filter = gtk_check_button_new_with_label (_(
		"Filter out hits not meant for your machine"));
	gtk_tooltips_set_tip (Firestarter.ttips, sep_filter, _(
		"Check this and the hits list will not show hits "
		"that have a destination field other than your IP."), "");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (sep_filter),
		gnome_config_get_bool ("/firestarter/Program/sep_filter=FALSE"));
	gtk_box_pack_start (GTK_BOX (vbox), sep_filter, FALSE, FALSE, 0);

	hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

	g_signal_connect_object (G_OBJECT (redundant_filter), "toggled",
		G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);
	g_signal_connect_object (G_OBJECT (sep_filter), "toggled",
		G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);

	return (table);
}

static GtkWidget*
create_advanced_page (GtkWidget *pbox)
{
	GtkWidget *frame;
	GtkWidget *table;
	GtkWidget *vbox;
	GtkWidget *label;

	table = gtk_table_new (3, 1, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE (table), 0);
	gtk_table_set_col_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);

	vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
	gtk_table_attach (GTK_TABLE (table), vbox, 0, 1, 0, 1,
			  GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);

	label = gtk_label_new (_(
		"The advanced settings let the experienced user fine tune\n"
		"the program operation.\n"));
	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);

	blockbroadcast = gtk_check_button_new_with_label (_("Block broadcast messages"));
	gtk_tooltips_set_tip (Firestarter.ttips, blockbroadcast, _(
		"If you enable this, Firestarter will disregard all network broadcast traffic."), "");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (blockbroadcast),
		gnome_config_get_bool ("/firestarter/Wizard/blockbroadcast=TRUE"));
	gtk_box_pack_start (GTK_BOX (vbox), blockbroadcast, FALSE, FALSE, 0);

	frame = gtk_frame_new (_("Preferred packet rejection method"));
	gtk_table_attach (GTK_TABLE (table), frame, 0, 1, 1, 2,
			  GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);

	vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
	gtk_container_add (GTK_CONTAINER (frame), vbox);

	selectdeny = gtk_radio_button_new_with_label (NULL, _("Deny"));
	gtk_tooltips_set_tip (Firestarter.ttips, selectdeny, _(
		"Silently ignore the rejected packets."), "");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (selectdeny),
		gnome_config_get_bool ("/firestarter/Wizard/selectdeny=FALSE"));
	gtk_box_pack_start (GTK_BOX (vbox), selectdeny, FALSE, FALSE, 0);

	selectreject = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (selectdeny),
		_("Reject"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (selectreject),         
		!gnome_config_get_bool ("/firestarter/Wizard/selectdeny=FALSE"));
	gtk_tooltips_set_tip (Firestarter.ttips, selectreject, _(
		"Reply with a notice of rejection to the sender."), "");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (selectreject),
		gnome_config_get_bool ("/firestarter/Wizard/selectreject=FALSE"));
	gtk_box_pack_start (GTK_BOX (vbox), selectreject, FALSE, FALSE, 0);


	g_signal_connect_object (G_OBJECT (blockbroadcast), "toggled",
		G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);
 
	g_signal_connect_object (G_OBJECT (selectdeny), "toggled",
		G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);

	g_signal_connect_object (G_OBJECT (selectreject), "toggled",
		G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);

	return table;
}

static void
prefs_select_row_cb (GtkCList *clist,
                     gint row,
                     gint column,
                     GdkEventButton *event,
                     GtkWidget *pbox)
{
        gtk_notebook_set_page (GTK_NOTEBOOK (
		GNOME_PROPERTY_BOX (pbox)->notebook), row);
}

static int
prefs_add_section_page (gchar     *name,
                        GtkWidget *widget,
			GtkWidget *pbox)
{
	gchar *text[2] = {name, NULL};
	gtk_notebook_append_page (GTK_NOTEBOOK (GNOME_PROPERTY_BOX (pbox)->notebook),
				  widget, gtk_label_new (name));
	return gtk_clist_append (GTK_CLIST (sections_clist), text);
}

static void
preferences_show_help (GnomePropertyBox *property_box, gint page_num)
{
	gnome_url_show ("http://firestarter.sourceforge.net/manual/preferences.php", NULL);
}

/* [ preferences_cb ]
 * Create the preferences property_box
 */
void
preferences_cb (GtkWidget *widget, gpointer data)
{
	static GtkWidget *pbox;
	GtkWidget *hbox;

	if (pbox != NULL) {
		g_assert (GTK_WIDGET_REALIZED (pbox));
		gdk_window_show (pbox->window);
		gdk_window_raise (pbox->window);

		return;
	}

	pbox = gnome_property_box_new ();
	gtk_window_set_title (GTK_WINDOW (pbox), _("Firestarter: Preferences"));

	hbox = gtk_hbox_new (FALSE, GNOME_PAD);
	gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);

	sections_clist = gtk_clist_new (1);
	gtk_clist_set_selection_mode (GTK_CLIST (sections_clist),
				      GTK_SELECTION_BROWSE);

	g_signal_connect (G_OBJECT (sections_clist), "select_row",
			  G_CALLBACK (prefs_select_row_cb), pbox);

	prefs_add_section_page (_("General"), create_general_page (pbox), pbox);
	prefs_add_section_page (_("Filters"), create_filters_page (pbox), pbox);
	
	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (GNOME_PROPERTY_BOX (pbox)->notebook), FALSE);
	gtk_notebook_set_show_border (GTK_NOTEBOOK (GNOME_PROPERTY_BOX (pbox)->notebook), FALSE);

	gtk_widget_ref (GNOME_PROPERTY_BOX (pbox)->notebook);

	gtk_container_remove (GTK_CONTAINER (GNOME_DIALOG (pbox)->vbox),
			      GNOME_PROPERTY_BOX (pbox)->notebook);

	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG(pbox)->vbox),
			    hbox, TRUE, TRUE, 0);

	gtk_box_pack_start (GTK_BOX (hbox),
			    sections_clist,
			    TRUE, TRUE, 0);

	gtk_box_pack_start (GTK_BOX (hbox),
			    GNOME_PROPERTY_BOX (pbox)->notebook,
			    TRUE, TRUE, 0);

	/* Import pages from wizard */
	wizard = g_new (Wizard, 1);
	prefs_add_section_page (_("External Device"), create_device_page (wizard), pbox);
	prefs_add_section_page (_("Connection Sharing"), create_masq_page (wizard), pbox);
	prefs_add_section_page (_("Services"), create_services_page (wizard), pbox);
	prefs_add_section_page (_("ICMP Filtering"), create_icmp_page (wizard), pbox);
	prefs_add_section_page (_("Type of Service"), create_tos_page (wizard), pbox);

	/* Widgets imported from wizard that can affect the script must be registered
	   with the property box */

	g_signal_connect_object (G_OBJECT (wizard->extdevice), "changed", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->pppcheck), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->dhcpcheck), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	


	g_signal_connect_object (G_OBJECT (wizard->masq), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->auto_int_ip), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->intdevice), "changed", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->intrange), "changed", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);

	g_signal_connect_object (G_OBJECT (wizard->services), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->ftp), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->ssh), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->telnet), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->smtp), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->dns), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->finger), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->www), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->sslweb), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->pop), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->imap), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->ident), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->nntp), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->ntp), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->samba), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->ipsec), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->routed), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->nfs), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->x), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->dhcp), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->upnp), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	

	g_signal_connect_object (G_OBJECT (wizard->icmp), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->icmp_echo), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->icmp_traceroute), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->icmp_tracert), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->icmp_unreach), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->icmp_timestamp), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->icmp_masking), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->icmp_redir), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->icmp_quench), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	

	g_signal_connect_object (G_OBJECT (wizard->tos), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->tos_client), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->tos_server), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->tos_X), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->tos_option_throughput), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->tos_option_reliability), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);	
	g_signal_connect_object (G_OBJECT (wizard->tos_option_delay), "toggled", G_CALLBACK (gnome_property_box_changed), G_OBJECT (pbox), G_CONNECT_SWAPPED);
	prefs_add_section_page (_("Advanced"), create_advanced_page (pbox), pbox);

	gtk_clist_columns_autosize (GTK_CLIST (sections_clist) );

	gtk_widget_unref (GNOME_PROPERTY_BOX (pbox)->notebook);

	g_signal_connect (G_OBJECT (pbox), "apply",
	                  G_CALLBACK (save_preferences), NULL);

	g_signal_connect (G_OBJECT (pbox), "help",
	                  G_CALLBACK (preferences_show_help), NULL);

	g_signal_connect (G_OBJECT (pbox), "destroy",
			    G_CALLBACK (gtk_widget_destroyed), &pbox);

	load_choices (wizard);

	gtk_widget_show_all (pbox);
}
