/*---[ wizard-choices.c ]---------------------------------------------
 * Copyright (C) 2000 Tomas Junnonen (majix@sci.fi)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Functions dealing with saving/restoring the users wizard choices
 *--------------------------------------------------------------------*/

#include "wizard-choices.h"
#include "wizard.h"
#include "util.h"

void
load_choices (Wizard *data)
{
	/* Internal/External devices */
	gtk_entry_set_text (GTK_ENTRY (data->extdevice),
		gnome_config_get_string ("/firestarter/Wizard/extdevice"));

	gtk_entry_set_text (GTK_ENTRY (data->intdevice),
		gnome_config_get_string ("/firestarter/Wizard/intdevice"));

	/* Internal network IP range (manual setting) */
	gnome_config_set_string ("/firestarter/Wizard/intrange",
		gtk_entry_get_text (GTK_ENTRY (data->intrange)));

	/* Autodetect internal network IP range */
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->auto_int_ip),
		gnome_config_get_bool ("/firestarter/Wizard/auto_int_ip"));

	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->pppcheck),
		gnome_config_get_bool ("/firestarter/Wizard/pppcheck"));

	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->dhcpcheck),
		gnome_config_get_bool ("/firestarter/Wizard/dhcpcheck"));

	/* Sensitivity toggles for the pages */
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->services),
		gnome_config_get_bool ("/firestarter/Wizard/services"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->icmp),
		gnome_config_get_bool ("/firestarter/Wizard/icmp"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->tos),
		gnome_config_get_bool ("/firestarter/Wizard/tos"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->masq),
		gnome_config_get_bool ("/firestarter/Wizard/masq"));

	/* Services */
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->ftp),
		gnome_config_get_bool ("/firestarter/Wizard/ftp"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->ssh),
		gnome_config_get_bool ("/firestarter/Wizard/ssh"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->telnet),
		gnome_config_get_bool ("/firestarter/Wizard/telnet"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->smtp),
		gnome_config_get_bool ("/firestarter/Wizard/smtp"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->dns),
		gnome_config_get_bool ("/firestarter/Wizard/dns"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->finger),
		gnome_config_get_bool ("/firestarter/Wizard/finger"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->www),
		gnome_config_get_bool ("/firestarter/Wizard/www"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->sslweb),
		gnome_config_get_bool ("/firestarter/Wizard/sslweb"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->pop),
		gnome_config_get_bool ("/firestarter/Wizard/pop"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->imap),
		gnome_config_get_bool ("/firestarter/Wizard/imap"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->ident),
		gnome_config_get_bool ("/firestarter/Wizard/ident"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->nntp),
		gnome_config_get_bool ("/firestarter/Wizard/nntp"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->ntp),
		gnome_config_get_bool ("/firestarter/Wizard/ntp"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->samba),
		gnome_config_get_bool ("/firestarter/Wizard/samba"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->ipsec),
		gnome_config_get_bool ("/firestarter/Wizard/ipsec"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->routed),
		gnome_config_get_bool ("/firestarter/Wizard/routed"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->nfs),
		gnome_config_get_bool ("/firestarter/Wizard/nfs"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->x),
		gnome_config_get_bool ("/firestarter/Wizard/x"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->dhcp),
		gnome_config_get_bool ("/firestarter/Wizard/dhcp"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->upnp),
		gnome_config_get_bool ("/firestarter/Wizard/upnp"));

	/* ICMP filtering */
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->icmp_echo),
		gnome_config_get_bool ("/firestarter/Wizard/icmp_echo"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->icmp_traceroute),
		gnome_config_get_bool ("/firestarter/Wizard/icmp_traceroute"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->icmp_tracert),
		gnome_config_get_bool ("/firestarter/Wizard/icmp_tracert"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->icmp_unreach),
		gnome_config_get_bool ("/firestarter/Wizard/icmp_unreach"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->icmp_timestamp),
		gnome_config_get_bool ("/firestarter/Wizard/icmp_timestamp"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->icmp_masking),
		gnome_config_get_bool ("/firestarter/Wizard/icmp_masking"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->icmp_redir),
		gnome_config_get_bool ("/firestarter/Wizard/icmp_redir"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->icmp_quench),
		gnome_config_get_bool ("/firestarter/Wizard/icmp_quench"));

	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->tos_client),
		gnome_config_get_bool ("/firestarter/Wizard/tos_client"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->tos_server),
		gnome_config_get_bool ("/firestarter/Wizard/tos_server"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->tos_X),
		gnome_config_get_bool ("/firestarter/Wizard/tos_X"));

	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->tos_option_throughput),
		gnome_config_get_bool ("/firestarter/Wizard/tos_option_throughput"));

	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->tos_option_reliability),
		gnome_config_get_bool ("/firestarter/Wizard/tos_option_reliability"));

	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->tos_option_delay),
		gnome_config_get_bool ("/firestarter/Wizard/tos_option_delay"));
}

void
save_choices (Wizard *data)
{
	/* Internal/External devices */
	gnome_config_set_string ("/firestarter/Wizard/extdevice",
		gtk_entry_get_text (GTK_ENTRY (data->extdevice)));
	gnome_config_set_string ("/firestarter/Wizard/intdevice",
		gtk_entry_get_text (GTK_ENTRY (data->intdevice)));

	/* Internal network IP range (manual setting) */
	gnome_config_set_string ("/firestarter/Wizard/intrange",
		gtk_entry_get_text (GTK_ENTRY (data->intrange)));
		
	/* Autodetect internal network IP range */
	gnome_config_set_bool ("/firestarter/Wizard/auto_int_ip",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->auto_int_ip)));

	gnome_config_set_bool ("/firestarter/Wizard/pppcheck",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->pppcheck)));
	gnome_config_set_bool ("/firestarter/Wizard/dhcpcheck",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->dhcpcheck)));

	/* Sensitivity toggles for the pages */
	gnome_config_set_bool ("/firestarter/Wizard/services",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->services)));
	gnome_config_set_bool ("/firestarter/Wizard/icmp",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->icmp)));
	gnome_config_set_bool ("/firestarter/Wizard/tos",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->tos)));
	gnome_config_set_bool ("/firestarter/Wizard/masq",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->masq)));

	/* Services */
	gnome_config_set_bool ("/firestarter/Wizard/ftp",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->ftp)));
	gnome_config_set_bool ("/firestarter/Wizard/ssh",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->ssh)));
	gnome_config_set_bool ("/firestarter/Wizard/telnet",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->telnet)));
	gnome_config_set_bool ("/firestarter/Wizard/smtp",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->smtp)));
	gnome_config_set_bool ("/firestarter/Wizard/dns",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->dns)));
	gnome_config_set_bool ("/firestarter/Wizard/finger",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->finger)));
	gnome_config_set_bool ("/firestarter/Wizard/www",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->www)));
	gnome_config_set_bool ("/firestarter/Wizard/sslweb",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->sslweb)));
	gnome_config_set_bool ("/firestarter/Wizard/pop",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->pop)));
	gnome_config_set_bool ("/firestarter/Wizard/imap",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->imap)));
	gnome_config_set_bool ("/firestarter/Wizard/ident",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->ident)));
	gnome_config_set_bool ("/firestarter/Wizard/nntp",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->nntp)));
	gnome_config_set_bool ("/firestarter/Wizard/ntp",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->ntp)));
	gnome_config_set_bool ("/firestarter/Wizard/samba",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->samba)));
	gnome_config_set_bool ("/firestarter/Wizard/ipsec",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->ipsec)));
	gnome_config_set_bool ("/firestarter/Wizard/routed",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->routed)));
	gnome_config_set_bool ("/firestarter/Wizard/nfs",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->nfs)));
	gnome_config_set_bool ("/firestarter/Wizard/x",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->x)));
	gnome_config_set_bool ("/firestarter/Wizard/dhcp",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->dhcp)));
	gnome_config_set_bool ("/firestarter/Wizard/upnp",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->upnp)));

	/* ICMP filtering */
	gnome_config_set_bool ("/firestarter/Wizard/icmp_echo",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->icmp_echo)));
	gnome_config_set_bool ("/firestarter/Wizard/icmp_traceroute",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->icmp_traceroute)));
	gnome_config_set_bool ("/firestarter/Wizard/icmp_tracert",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->icmp_tracert)));
	gnome_config_set_bool ("/firestarter/Wizard/icmp_unreach",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->icmp_unreach)));
	gnome_config_set_bool ("/firestarter/Wizard/icmp_timestamp",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->icmp_timestamp)));
	gnome_config_set_bool ("/firestarter/Wizard/icmp_masking",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->icmp_masking)));
	gnome_config_set_bool ("/firestarter/Wizard/icmp_redir",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->icmp_redir)));
	gnome_config_set_bool ("/firestarter/Wizard/icmp_quench",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->icmp_quench)));

	/* TOS filtering */
	gnome_config_set_bool ("/firestarter/Wizard/tos_client",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->tos_client)));
	gnome_config_set_bool ("/firestarter/Wizard/tos_server",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->tos_server)));
	gnome_config_set_bool ("/firestarter/Wizard/tos_X",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->tos_X)));

	gnome_config_set_bool ("/firestarter/Wizard/tos_option_throughput",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->tos_option_throughput)));
	gnome_config_set_bool ("/firestarter/Wizard/tos_option_reliability",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->tos_option_reliability)));
	gnome_config_set_bool ("/firestarter/Wizard/tos_option_delay",
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->tos_option_delay)));

	gnome_config_sync ();
}
