/*---[ service.c ]----------------------------------------------------
 * Copyright (C) 2000 Tomas Junnonen (majix@sci.fi)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Return the service used, based on the port and protocol given
 *--------------------------------------------------------------------*/

#include <netdb.h>
#include <netinet/in.h>

#include "service.h"

/* [ service ]
 * Return the service/exploit used, based on the port and protocol given
 */
gchar *
get_service_name (gint port, gchar *proto)
{
	struct servent *ent;

	if (port == 0 || proto == NULL)
		return (g_strdup ("unknown"));

	if (port == 135) return (g_strdup ("ms-rpc"));
	if (port == 31337) return (g_strdup ("borifice"));
	if (port == 12345) return (g_strdup ("netbus"));
	if (port == 12346) return (g_strdup ("netbus"));
	if (port == 1524) return (g_strdup ("trin00"));
	if (port == 27665) return (g_strdup ("trin00"));
	if (port == 27444) return (g_strdup ("trin00"));
	if (port == 31335) return (g_strdup ("trin00"));
	if (port == 33270) return (g_strdup ("trinity"));
	if (port == 1234) return (g_strdup ("subseven"));
	if (port == 5000) return (g_strdup ("uPNP"));
	if (port == 6711) return (g_strdup ("subseven"));
	if (port == 16660) return (g_strdup ("stacheldraht"));
	if (port == 60001) return (g_strdup ("stacheldraht"));

	ent = getservbyport (htons (port), proto);

	if (ent)
		return (g_strdup (ent->s_name));
	else
		return (g_strdup ("unknown"));
}

gchar *
get_service_name_any_protocol (gint port) {
	gchar *name = NULL;

	name = get_service_name (port, "tcp");
	if (strcmp (name, "unknown") == 0)
		name = get_service_name (port, "udp");
		
	return name;
}
